package com.oqpo.api.service.pomng;


import com.oqpo.api.entity.loanmng.LoanMgntBaseInfoEntity;
import com.oqpo.api.entity.oper.UserMngEntity;
import com.oqpo.api.entity.pomng.PchOdrBaseInfoEntity;
import com.oqpo.api.entity.pomng.PchOdrDtlPtclEntity;
import com.oqpo.api.entity.pomng.PchReqBaseInfoEntity;
import com.oqpo.api.entity.pomng.PchReqDtlPtclEntity;
import com.oqpo.api.enums.SystemMessageCode;
import com.oqpo.api.exception.GlobalException;
import com.oqpo.api.mapper.loanmng.LoanStateMapper;
import com.oqpo.api.mapper.oper.UserMngMapper;
import com.oqpo.api.mapper.pomng.PchOrdMapper;
import com.oqpo.api.mapper.pomng.PchReqMapper;
import com.oqpo.api.service.CommonService;
import com.oqpo.api.util.DateUtil;
import com.oqpo.api.web.dto.request.GridRequest;
import com.oqpo.api.web.dto.request.pomng.pchOrd.PchReqOrdInitRequest;
import com.oqpo.api.web.dto.request.pomng.pchOrd.SavePchOrdRequest;
import com.oqpo.api.web.dto.request.pomng.pchReq.SavePchReqlRequest;
import com.oqpo.api.web.dto.response.GridResponse;
import com.oqpo.api.web.dto.response.pomng.pchOrd.PchOrdInitInfoResponse;
import com.oqpo.api.web.dto.response.pomng.pchOrd.PoOrdInitInfoResponse;
import com.oqpo.api.web.dto.response.pomng.pchReq.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Service
@Slf4j
public class PchOrdService extends CommonService {
    @Autowired
    private PchOrdMapper pchOrdMapper;

    @Autowired
    private UserMngMapper userMngMapper;


    /* 구매요청건 발주 가능 여부 체크 */
    public String  chkPchReqUseOrd(String userId, PchReqOrdInitRequest pchReqOrdInitRequest ) throws Exception {
        String usePassPoYn = "Y";
        // 초기 발주 head 정보 가져오기
        List<PchOdrBaseInfoEntity> headList = null;
        List<PchOdrDtlPtclEntity> itemList = null;

        // 품목 데이타 확인 필요
        List<PchOdrDtlPtclEntity> gridUpdateData = pchReqOrdInitRequest.toCodeEntities(pchReqOrdInitRequest.getGridUpdateData());
        if (gridUpdateData.size() > 0) {
            headList = pchOrdMapper.selectRecPchInitInfo(pchReqOrdInitRequest.getBrandId(),gridUpdateData);
            itemList = pchOrdMapper.selectRecPchDtlInitInfo(pchReqOrdInitRequest.getBrandId(),gridUpdateData);
        }

        if (headList.size() > 1) {
            // 서로다른 납품장소로 인한 처리 불가 오류 메세지 출력
            usePassPoYn = "N";
            throw new GlobalException(SystemMessageCode.ERR_NEQWHSID);
        }

        PchOdrBaseInfoEntity headData =  headList.get(0);
        String dlvReqDt = ""; // 납품요청일
        String spplyId = ""; // 공급업체

        if ("DL01".equals(headData.getDlvDvsn())) {
              // 본사 배송은 매장으로의 배송이며, 납품장소는 매장이어야 함
              // 구매요청건이 같은 매장이어야 하며, 납품요청일 동일해야함
            if ("".equals(headData.getStoreId())) {
                usePassPoYn = "N";
                throw new GlobalException(SystemMessageCode.ERR_NONSTORE);
            }

            for (int j = 0 ; j < itemList.size() ; j++) {
                PchOdrDtlPtclEntity itemInfo = itemList.get(j);
                if ("".equals(dlvReqDt)) {
                    dlvReqDt = itemInfo.getDlvReqDt();
                } else {
                    if (!dlvReqDt.equals(itemInfo.getDlvReqDt())) {
                        usePassPoYn = "N";
                        throw new GlobalException(SystemMessageCode.ERR_NEQDLVDT);
                    }
                }
            }
        } else if ("DL02".equals(headData.getDlvDvsn())) {
            // 공급사 배송
            for (int i = 0; i < itemList.size(); i++) {
                PchOdrDtlPtclEntity itemInfo = itemList.get(i);
                if ("".equals(spplyId)) {
                    spplyId = itemInfo.getSpplyId();
                } else {
                    if (!spplyId.equals(itemInfo.getSpplyId())) {
                        usePassPoYn = "N";
                        throw new GlobalException(SystemMessageCode.ERR_NEQSPPLY);
                    }
                }
                if ("".equals(spplyId)) {
                    usePassPoYn = "N";
                    throw new GlobalException(SystemMessageCode.ERR_NONSPPLY);
                }

            }
        }

        return usePassPoYn;
    }


    /* 구매요청 발주  초기 정보
    *
    * 1. 구매요청에서 선택된 품목에 대해 발주 대상 품목 가능 여부 - 납품장소가 매장이면  납품요청일 + , 납품장소가 본사창고이면 납품요청일 + 납품공급사 ㅇ여부 체크하고 이상없을시
    *     발주생성화면으로 전환
    * 2. 발주 초기데이타에 대해 한번더 점검한후 발주생성 --
    * */

    public PchOrdInitInfoResponse selectRecPchInitInfo(String userId, PchReqOrdInitRequest pchReqOrdInitRequest ) throws Exception {

        // 로그인 정보 가져오기
        UserMngEntity uEntity = userMngMapper.selectGroupUserInfo(userId);

        // 초기 발주 head 정보 가져오기
        List<PchOdrBaseInfoEntity> headList = null;
        List<PchOdrDtlPtclEntity> itemList = null;

        // 품목 데이타 확인 필요
        List<PchOdrDtlPtclEntity> gridUpdateData = pchReqOrdInitRequest.toCodeEntities(pchReqOrdInitRequest.getGridUpdateData());
        if (gridUpdateData.size() > 0) {
            headList = pchOrdMapper.selectRecPchInitInfo(pchReqOrdInitRequest.getBrandId(),gridUpdateData);
            itemList = pchOrdMapper.selectRecPchDtlInitInfo(pchReqOrdInitRequest.getBrandId(),gridUpdateData);
        }

        if (headList.size() > 1) {
            // 서로다른 납품장소로 인한 처리 불가 오류 메세지 출력
            throw new GlobalException(SystemMessageCode.ERR_NEQWHSID);
        }

        PchOdrBaseInfoEntity headData =  headList.get(0);
        int podrTotalAmt = 0 ; // 총 발주금액
        String dlvReqDt = ""; // 납품요청일
        String spplyId = ""; // 공급업체
        String spplyNm = ""; //공급업체명

        if ("DL01".equals(headData.getDlvDvsn())) {

            // 매장 배송
            if ("".equals(headData.getStoreId())) {
                throw new GlobalException(SystemMessageCode.ERR_NONSTORE);
            }

            for (int j = 0 ; j < itemList.size() ; j++) {
                PchOdrDtlPtclEntity itemInfo = itemList.get(j);
                podrTotalAmt = podrTotalAmt + (itemInfo.getUnitAmt() * itemInfo.getPchReqQty());
                if ("".equals(dlvReqDt)) {
                    dlvReqDt = itemInfo.getDlvReqDt();
                } else {
                    if (!dlvReqDt.equals(itemInfo.getDlvReqDt())) {
                        throw new GlobalException(SystemMessageCode.ERR_NEQDLVDT);
                    }
                }
            }
        } else if ("DL02".equals(headData.getDlvDvsn())) {
            //본사배송 - 동일 공급사 품목이어야 함
            // 공급사 배송
            for (int i = 0 ; i < itemList.size() ;i++) {
                PchOdrDtlPtclEntity itemInfo = itemList.get(i);
                podrTotalAmt = podrTotalAmt + (itemInfo.getUnitAmt() * itemInfo.getPchReqQty());
                if ("".equals(spplyId)) {
                    spplyId = itemInfo.getSpplyId();
                    spplyNm = itemInfo.getSpplyNm();
                } else {
                    if (!spplyId.equals(itemInfo.getSpplyId())) {
                        throw new GlobalException(SystemMessageCode.ERR_NEQSPPLY);
                    }
                }
                if ("".equals(spplyId)) {
                    throw new GlobalException(SystemMessageCode.ERR_NONSPPLY);
                }
            }
        }

        headData.setSpplyId(spplyId);
        headData.setSpplyNm(spplyNm);
        headData.setPodrTotalAmt(podrTotalAmt);


        return PchOrdInitInfoResponse.toDTO(uEntity, headData, itemList);
    }

    /*
    발주 신규생성 초기데이타
     */
    public PoOrdInitInfoResponse selectPoInitInfo(String userId, String brandId) throws Exception {

        // 로그인 정보 가져오기
        UserMngEntity uEntity = userMngMapper.selectGroupUserInfo(userId);

        // 초기 발주 head 정보 가져오기
        PchOdrBaseInfoEntity headData = pchOrdMapper.selectPoInitInfo(brandId);

        return PoOrdInitInfoResponse.toDTO(uEntity, headData);
    }

    /*
      발주 정보 저장
     */
    @Transactional
    public void savePchOrdInfo(String userId, SavePchOrdRequest savePchOrdRequest) throws Exception {
        try {

            if ("C".equals(savePchOrdRequest.getViewCd())) {
                // 구매발주 아이디 부여
                //entity.setPchReqUnqNo(fnGetDealNo(14,""));  // 구매발주번호 생성
            }

            List<PchOdrBaseInfoEntity> headList = null;
            List<PchOdrDtlPtclEntity> itemList = null;
            if ("R".equals(savePchOrdRequest.getJobCd())) {   // 구매요청건
                // 발주상세 정보를 세팅한다.
                /*
                List<PchOdrDtlPtclEntity> gridUpdateData = savePchOrdRequest.toCodeEntities(savePchOrdRequest.getGridInsertData());
                if (gridUpdateData.size() > 0) {
                    headList = pchOrdMapper.selectRecPchInitInfo(savePchOrdRequest.getBrandId(),gridUpdateData);
                    itemList = pchOrdMapper.selectRecPchDtlInitInfo(savePchOrdRequest.getBrandId(),gridUpdateData);
                }

                if (headList.size() > 1) {
                    // 서로다른 납품장소로 인한 처리 불가 오류 메세지 출력
                    throw new GlobalException(SystemMessageCode.ERR_NEQWHSID);
                }
                PchOdrBaseInfoEntity headData =  headList.get(0);

                if ("DL01".equals(headData.getDlvDvsn())) {
                    // 본사 배송은 매장으로의 배송이며, 납품장소는 매장이어야 함
                    // 구매요청건이 같은 매장이어야 하며, 납품요청일 동일해야함
                    if ("".equals(headData.getStoreId())) {
                        throw new GlobalException(SystemMessageCode.ERR_NONSTORE);
                    }


                } else if ("DL02".equals(headData.getDlvDvsn())) {
                    // 공급사 배송
                    for (int i = 0; i < itemList.size(); i++) {
                        PchOdrDtlPtclEntity itemInfo = itemList.get(i);
                        if ("".equals(spplyId)) {
                            spplyId = itemInfo.getSpplyId();
                        } else {
                            if (!spplyId.equals(itemInfo.getSpplyId())) {
                                usePassPoYn = "N";
                                throw new GlobalException(SystemMessageCode.ERR_NEQSPPLY);
                            }
                        }
                        if ("".equals(spplyId)) {
                            usePassPoYn = "N";
                            throw new GlobalException(SystemMessageCode.ERR_NONSPPLY);
                        }

                    }
                }



        */

            } else if ("P".equals(savePchOrdRequest.getJobCd())) {  //신규발주건

            }


        } catch (GlobalException e) {
            e.getStackTrace();
            throw new GlobalException(e.getSystemMessageCode());
        } catch (Exception e) {
            e.getStackTrace();
            throw new RuntimeException();
        }
    }
}
