package com.oqpo.api.web.controller.pomng;

import com.oqpo.api.enums.DataMessageCode;
import com.oqpo.api.enums.SystemMessageCode;
import com.oqpo.api.service.pomng.PchOrdService;
import com.oqpo.api.service.pomng.PchReqService;
import com.oqpo.api.web.dto.request.pomng.pchOrd.PchReqOrdInitRequest;
import com.oqpo.api.web.dto.request.pomng.pchOrd.SavePchOrdRequest;
import com.oqpo.api.web.dto.request.pomng.pchReq.PchReqPoTargetSearchGridRequest;
import com.oqpo.api.web.dto.request.pomng.pchReq.PchReqSearchGridRequest;
import com.oqpo.api.web.dto.request.pomng.pchReq.SavePchReqlRequest;
import com.oqpo.api.web.dto.response.DataResponse;
import com.oqpo.api.web.dto.response.GridResponse;
import com.oqpo.api.web.dto.response.SaveResponse;
import com.oqpo.api.web.dto.response.pomng.pchOrd.PchOrdInitInfoResponse;
import com.oqpo.api.web.dto.response.pomng.pchOrd.PoOrdInitInfoResponse;
import com.oqpo.api.web.dto.response.pomng.pchReq.PchReqInfoResponse;
import com.oqpo.api.web.dto.response.pomng.pchReq.PchReqInitInfoResponse;
import com.oqpo.api.web.dto.response.pomng.pchReq.PchReqRejectTargetResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;

@Slf4j
@RestController
@RequestMapping("/api/pomng")
@Api(tags = {"구매발주 관리 (PchOrdController) -- 진행중"})
public class PchOrdController {
    @Autowired
    private PchOrdService pchOrdService;

    //
    /**
     * 설명 : 구매요청 발주 생성 가능 여부 체크
     *
     * @param userId
     * @return
     * @throws Exception
     */
    @ApiImplicitParams({
            @ApiImplicitParam(name = "X-AUTH-TOKEN", value = "CONN-KEY", required = true, dataType = "String", paramType = "header")
    })
    @ApiOperation(value = "구매요청 발주가능 여부 체크")
    @GetMapping("/chk-pchOrd")
    public ResponseEntity<DataResponse> chkPchReqUseOrd(@ApiIgnore String userId, @RequestBody @Valid PchReqOrdInitRequest pchReqOrdInitRequest) throws Exception {
        String chkUsePo = pchOrdService.chkPchReqUseOrd(userId, pchReqOrdInitRequest);
        DataMessageCode message = null;
        if ("Y".equals(chkUsePo)) {
            message = DataMessageCode.OK_PCHORD;
        } else {
            message = DataMessageCode.NOT_NONPO;
        }
        return ResponseEntity.ok(DataResponse.toDTO(message));
    }

    /**
     * 설명 : 구매요청 발주생성  초기 정보
     *
     * @param userId
     * @return
     * @throws Exception
     */
    @ApiImplicitParams({
            @ApiImplicitParam(name = "X-AUTH-TOKEN", value = "CONN-KEY", required = true, dataType = "String", paramType = "header")
    })
    @ApiOperation(value = "구매요청 발주생성  초기 정보")
    @GetMapping("/init-pchOrd")
    public ResponseEntity<PchOrdInitInfoResponse> selectPchOrdInitInfo(@ApiIgnore String userId, @RequestBody @Valid PchReqOrdInitRequest pchReqOrdInitRequest) throws Exception {
        return ResponseEntity.ok(pchOrdService.selectRecPchInitInfo(userId, pchReqOrdInitRequest));
    }

    /**
     * 설명 : 구매요청 발주생성  초기 정보
     *
     * @param userId
     * @return
     * @throws Exception
     */
    @ApiImplicitParams({
            @ApiImplicitParam(name = "X-AUTH-TOKEN", value = "CONN-KEY", required = true, dataType = "String", paramType = "header"),
            @ApiImplicitParam(name = "brandId", value = "브랜드아이디", required = true, dataType = "String", paramType = "query")
    })
    @ApiOperation(value = "신규발주생성  초기 정보")
    @GetMapping("/init-poOrd")
    public ResponseEntity<PoOrdInitInfoResponse> selectPoOrdInitInfo(@ApiIgnore String userId,@RequestParam(value = "brandId") String brandId) throws Exception {
        return ResponseEntity.ok(pchOrdService.selectPoInitInfo(userId, brandId));
    }

    /**
     * 설명 : 구매발주 정보 저장
     *
     * @param userId
     * @param savePchReqlRequest
     * @return SaveResponse
     * @throws Exception
     */

    @ApiImplicitParams({
            @ApiImplicitParam(name = "X-AUTH-TOKEN", value = "CONN-KEY", required = true, dataType = "String", paramType = "header")
    })
    @ApiOperation(value = "구매발주 정보 저장")
    @PostMapping("/save-pchPo")
    public ResponseEntity<SaveResponse> savePchReq(@ApiIgnore String userId, @RequestBody @Valid SavePchOrdRequest savePchOrdRequest) throws Exception {
        pchOrdService.savePchOrdInfo(userId,  savePchOrdRequest);
        return ResponseEntity.ok(SaveResponse.toDTO(SystemMessageCode.SAVE_OK));
    }

    /**
     * 설명 : 구매요청 그리드 리스트
     *
     * @param pchReqSearchGridRequest
     * @return
     * @throws Exception
     */
/*
    @ApiImplicitParams({
            @ApiImplicitParam(name = "X-AUTH-TOKEN", value = "CONN-KEY", required = true, dataType = "String", paramType = "header")
    })
    @ApiOperation(value = " 구매요청 그리드 리스트")
    @PostMapping("/detail-grid-list")
    public ResponseEntity<GridResponse> detailGridList(@ApiIgnore String userId,   @RequestBody @Valid PchReqSearchGridRequest pchReqSearchGridRequest) throws Exception {
        return ResponseEntity.ok(pchReqService.selectPchReqGridList(userId, pchReqSearchGridRequest.getSBrandId(), pchReqSearchGridRequest.getSStoreId(), pchReqSearchGridRequest.getFromDt(), pchReqSearchGridRequest.getToDt(), pchReqSearchGridRequest.getSWhsId(), pchReqSearchGridRequest.getSPchReqStCd(), pchReqSearchGridRequest.getSItemNm(),  pchReqSearchGridRequest.toDTO(pchReqSearchGridRequest)));
    }
*/

    /**
     * 설명 : 구매요청 정보
     *
     * @param brandId, pchReqUnqNo
     * @return
     * @throws Exception
     */
    /*
    @ApiImplicitParams({
            @ApiImplicitParam(name = "X-AUTH-TOKEN", value = "CONN-KEY", required = true, dataType = "String", paramType = "header"),
            @ApiImplicitParam(name = "brandId", value = "브랜드아이디", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "pchReqUnqNo", value = "구매요청번호", required = true, dataType = "String", paramType = "query")
    })
    @ApiOperation(value = "구매요청 정보")
    @GetMapping("/info-pchReq")
    public ResponseEntity<PchReqInfoResponse> infoPchReq(@RequestParam(value = "brandId") String brandId, @RequestParam(value = "pchReqUnqNo") String pchReqUnqNo) throws Exception {
        return ResponseEntity.ok(pchReqService.selectPchReqInfo(brandId,pchReqUnqNo));
    }
*/

    /**
     * 설명 : 구매요청 정보 저장
     *
     * @param userId
     * @param savePchReqlRequest
     * @return SaveResponse
     * @throws Exception
     */
    /*
    @ApiImplicitParams({
            @ApiImplicitParam(name = "X-AUTH-TOKEN", value = "CONN-KEY", required = true, dataType = "String", paramType = "header")
    })
    @ApiOperation(value = "구매요청 정보 저장")
    @PostMapping("/save-pchReq")
    public ResponseEntity<SaveResponse> savePchReq(@ApiIgnore String userId, @RequestBody @Valid SavePchReqlRequest savePchReqlRequest) throws Exception {
        pchReqService.savePchReqInfo(userId,  savePchReqlRequest);
        return ResponseEntity.ok(SaveResponse.toDTO(SystemMessageCode.SAVE_OK));
    }
*/
    /**
     * 설명 : 구매발주 대상 그리드 리스트
     *
     * @param pchReqSearchGridRequest
     * @return
     * @throws Exception
     */
/*
    @ApiImplicitParams({
            @ApiImplicitParam(name = "X-AUTH-TOKEN", value = "CONN-KEY", required = true, dataType = "String", paramType = "header")
    })
    @ApiOperation(value = " 구매발주 대상 그리드 리스트")
    @PostMapping("/po-target-grid-list")
    public ResponseEntity<GridResponse> detailGridList(@RequestBody @Valid PchReqPoTargetSearchGridRequest pchReqPoTargetSearchGridRequest) throws Exception {
        return ResponseEntity.ok(pchReqService.selectPchReqPoTargetGridList(pchReqPoTargetSearchGridRequest.getSBrandId(), pchReqPoTargetSearchGridRequest.getFromDt(), pchReqPoTargetSearchGridRequest.getToDt(), pchReqPoTargetSearchGridRequest.getSSpplyId(), pchReqPoTargetSearchGridRequest.getSWhsId(), pchReqPoTargetSearchGridRequest.getSItemNm(),  pchReqPoTargetSearchGridRequest.toDTO(pchReqPoTargetSearchGridRequest)));
    }
*/

    /**
     * 설명 : 구매요청 반련 대상 리스트
     *
     * @param pchReqRejectTargerListRequest
     * @return
     * @throws Exception
     */
    /*
    @ApiImplicitParams({
            @ApiImplicitParam(name = "X-AUTH-TOKEN", value = "CONN-KEY", required = true, dataType = "String", paramType = "header"),
            @ApiImplicitParam(name = "pchReqUnqNo", value = "구매요청번호", required = true, dataType = "String", paramType = "query")
    })
    @ApiOperation(value = "구매요청 반련 대상 리스트")
    @GetMapping("/reject-pchreq-target")
    public ResponseEntity<PchReqRejectTargetResponse> rejectTargetPchReq(@RequestParam(value = "pchReqUnqNo") String pchReqUnqNo) throws Exception {

            return ResponseEntity.ok(pchReqService.selectPchRejectPrTargetList(pchReqUnqNo));
    }
*/

    /**
     * 설명 : 구매요청 반려 처리
     *
     * @param userId
     * @param savePchReqlRequest
     * @return SaveResponse
     * @throws Exception
     */
    /*
    @ApiImplicitParams({
            @ApiImplicitParam(name = "X-AUTH-TOKEN", value = "CONN-KEY", required = true, dataType = "String", paramType = "header"),
            @ApiImplicitParam(name = "pchReqUnqNo", value = "구매요청번호", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "pchReqRjctRsn", value = "반려사유", required = true, dataType = "String", paramType = "query"),
    })
    @ApiOperation(value = "구매요청 반려 처리")
    @GetMapping("/reject-pchReq")
    public ResponseEntity<SaveResponse> rejectPchReq(@ApiIgnore String userId, @RequestParam(value = "pchReqUnqNo") String pchReqUnqNo, @RequestParam(value = "pchReqRjctRsn") String pchReqRjctRsn) throws Exception {
        pchReqService.saveRejectPchReqInfo(userId,  pchReqUnqNo, pchReqRjctRsn);
        return ResponseEntity.ok(SaveResponse.toDTO(SystemMessageCode.SAVE_OK));
    }
*/

}
